/* include/soc/samsung/asv-exynos3250.h
*
* Copyright (c) 2012 Samsung Electronics Co., Ltd.
*              http://www.samsung.com/
*
* EXYNOS3250 - Adoptive Support Voltage Header file
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*/

#ifndef __ASM_ARCH_EXYNOS3250_ASV_H
#define __ASM_ARCH_EXYNOS3250_ASV_H __FILE__

#define ARM_DVFS_LEVEL_NR		(10)
#define ARM_ASV_GRP_NR			(13)
#define ARM_MAX_VOLT			(1150000)

#define INT_DVFS_LEVEL_NR		(6)
#define INT_ASV_GRP_NR			(13)
#define INT_MAX_VOLT			(950000)

#define MIF_DVFS_LEVEL_NR		(5)
#define MIF_ASV_GRP_NR			(13)
#define MIF_MAX_VOLT			(875000)

static unsigned int refer_table_get_asv[2][ARM_ASV_GRP_NR] = {
	/*0   1   2   3   4   5   6   7   8   9  10  11  12  13  14*/
	{ 0,  2,  2,  3,  5,  8, 11, 15, 20, 26, 33, 44, 999},
	{ 0, 17, 19, 24, 27, 30, 32, 34, 36, 38, 40, 42, 999},
};
/* ABB setting value */
static unsigned int arm_asv_abb_info[ARM_DVFS_LEVEL_NR][ARM_ASV_GRP_NR + 1] = {
	/*  FREQ       ASV0       ASV1         ASV2        ASV3        ASV4        ASV5        ASV6        ASV7        ASV8        ASV9       ASV10       ASV11       ASV12 */
	{ 1000000, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS },		/* L1 */
	{  900000, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS },		/* L2 */
	{  800000, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS },		/* L3 */
	{  700000, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS,	ABB_X130,   ABB_X130  },		/* L4 */
	{  600000, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS,	ABB_X130,   ABB_X130,   ABB_X130,   ABB_X130  },		/* L5 */
	{  500000, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,   ABB_X130,   ABB_X130  },		/* L6 */
	{  400000, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,   ABB_X130,   ABB_X130  },		/* L7 */
	{  300000, ABB_BYPASS, ABB_BYPASS, ABB_BYPASS,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,   ABB_X130,   ABB_X130  },		/* L8 */
	{  200000, ABB_X130,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,   ABB_X130,   ABB_X130,   ABB_X130,   ABB_X130  },		/* L9 */
	{  100000, ABB_X130,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,	ABB_X130,   ABB_X130,   ABB_X130,   ABB_X130,   ABB_X130,   ABB_X130  },		/* L10 */
};

/* ASV setting value */
static unsigned int arm_asv_volt_info[ARM_DVFS_LEVEL_NR][ARM_ASV_GRP_NR + 1] = {
	/*  FREQ      ASV0     ASV1     ASV2     ASV3     ASV4     ASV5     ASV6     ASV7     ASV8     ASV9    ASV10    ASV11    ASV12 */
	{ 1000000, 1150000, 1150000, 1125000, 1100000, 1075000, 1050000, 1025000, 1000000, 1000000,  975000,  975000,  950000,  950000 },	/* L0 */
	{  900000, 1112500, 1112500, 1087500, 1062500, 1037500, 1012500,  987500,  962500,  962500,  937500,  937500,  912500,  912500 },	/* L1 */
	{  800000, 1075000, 1075000, 1050000, 1025000, 1000000,  975000,  950000,  925000,  925000,  900000,  900000,  875000,  875000 },	/* L2 */
	{  700000, 1037500, 1037500, 1012500,  987500,  962500,  937500,  912500,  887500,  887500,  862500,  862500,  850000,  850000 },	/* L3 */
	{  600000, 1000000, 1000000,  975000,  950000,  925000,  900000,  875000,  850000,  850000,  850000,  850000,  850000,  850000 },	/* L4 */
	{  500000,  962500,  962500,  937500,  912500,  887500,  862500,  850000,  850000,  850000,  850000,  850000,  850000,  850000 },	/* L5 */
	{  400000,  925000,  925000,  900000,  875000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000 },	/* L6 */
	{  300000,  887500,  887500,  862500,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000 },	/* L7 */
	{  200000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000 },	/* L8 */
	{  100000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000,  850000 },	/* L9 */
};

static unsigned int int_asv_volt_info[INT_DVFS_LEVEL_NR][INT_ASV_GRP_NR + 1] = {
	/*  FREQ     ASV0     ASV1    ASV2    ASV3    ASV4    ASV5    ASV6    ASV7    ASV8    ASV9   ASV10   ASV11   ASV12  */
	{ 135000,  950000,  950000, 925000, 925000, 900000, 875000, 875000, 850000, 850000, 850000, 850000, 850000, 850000 },	/* L0 */
	{ 134000,  950000,  950000, 925000, 925000, 900000, 875000, 875000, 850000, 850000, 850000, 850000, 850000, 850000 },	/* L1 */
	{ 133000,  925000,  925000, 900000, 900000, 875000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000 },	/* L2 */
	{ 100000,  850000,  850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000 },	/* L3 */
	{  80000,  850000,  850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000 },	/* L4 */
	{  50000,  850000,  850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000, 850000 },	/* L5 */
};

static unsigned int mif_asv_volt_info[MIF_DVFS_LEVEL_NR][MIF_ASV_GRP_NR + 1] = {
	/*  FREQ    ASV0    ASV1    ASV2    ASV3    ASV4    ASV5    ASV6    ASV7    ASV8    ASV9   ASV10   ASV11   ASV12  */
	{ 400000, 875000, 875000, 850000, 825000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000 },	/* L0 */
	{ 200000, 825000, 825000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000 },	/* L1 */
	{ 133000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000 },	/* L2 */
	{ 100000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000 },	/* L3 */
	{  50000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000, 800000 },	/* L4 */
};

#endif /* EXYNOS3250_ASV_H */
